@@ -26,12 +26,12 @@ use XAO::Errors qw(XAO::DO::Web::Clipboard);
 use base XAO::Objects->load(objname => 'Web::Action');
 
 use vars qw($VERSION);
-($VERSION)=(q$Id: Clipboard.pm,v 1.2 2002/01/04 02:13:23 am Exp $ =~ /(\d+\.\d+)/);
+($VERSION)=(q$Id: Clipboard.pm,v 1.3 2002/05/19 20:00:01 am Exp $ =~ /(\d+\.\d+)/);
 
 sub check_mode ($$) {
     my $self = shift;
     my $args = get_args(\@_);
-    my $mode = $args->{mode};
+    my $mode = $args->{mode} || 'show';
 
     if ($mode eq 'show') {
         $self->clipboard_show($args);
