@@ -23,12 +23,14 @@ handler when site is initialized.
 ###############################################################################
 package XAO::DO::Web::Config;
 use XAO::Utils;
+use XAO::Cache;
 use XAO::Errors qw(XAO::DO::Web::Config);
 
 ##
 # Prototypes
 #
 sub add_cookie ($@);
+sub cache ($%);
 sub cgi ($$);
 sub cleanup ($);
 sub clipboard ($);
@@ -44,7 +46,7 @@ sub new ($@);
 # Package version for checks and reference
 #
 use vars qw($VERSION);
-($VERSION)=(q$Id: Config.pm,v 1.5 2002/01/04 03:27:25 am Exp $ =~ /(\d+\.\d+)/);
+($VERSION)=(q$Id: Config.pm,v 1.6 2002/05/17 05:19:03 am Exp $ =~ /(\d+\.\d+)/);
 
 ###############################################################################
 
@@ -106,6 +108,52 @@ sub add_cookie ($@) {
 
 ###############################################################################
 
+=item cache (%)
+
+Creates or retrieves a cache for use in various XAO::Web objects.
+Arguments are directly passed to XAO::Cache's new() method (see
+L<XAO::Cache>) except for 'name' argument which is used to identify the
+requested cache.
+
+If a cache with that name was already initialized before it is not
+re-created, but previously created version is returned instead.
+
+Example:
+
+ my $cache=$self->cache(
+     name        => 'fubar',
+     retrieve    => \&real_retrieve,
+     coords      => ['foo','bar'],
+     expire      => 60
+ );
+
+Caches are kept between executions in mod_perl environment.
+
+=cut
+
+sub cache ($%) {
+    my $self=shift;
+    my $args=get_args(\@_);
+
+    my $name=$args->{name} ||
+        throw XAO::E::DO::Web::Config "cache - no 'name' argument";
+
+    my $cache_list=$self->{cache_list};
+    if(! $cache_list) {
+        $cache_list=$self->{cache_list}={};
+    }
+
+    my $cache=$cache_list->{$name};
+    if(! $cache) {
+        $cache=XAO::Cache->new($args);
+        $cache_list->{$name}=$cache;
+    }
+
+    return $cache;
+}
+
+###############################################################################
+
 =item cgi (;$)
 
 Returns or sets standard CGI object (see L<CGI>). In future versions this
@@ -220,7 +268,7 @@ header(), header_args().
 =cut
 
 sub embeddable_methods ($) {
-    qw(add_cookie cgi clipboard cookies header header_args);
+    qw(add_cookie cache cgi clipboard cookies header header_args);
 }
 
 ###############################################################################
