@@ -8,7 +8,10 @@ Currently is only useful in XAO::Web site context.
 
 =head1 DESCRIPTION
 
-IdentifyUser class is used for user identification and verification purposes. In 'login' mode it logs a user in while in 'logout' mode, it logs a user out. In 'check' mode it determines the identification status of the user using cookies.
+IdentifyUser class is used for user identification and verification
+purposes. In 'login' mode it logs a user in while in 'logout' mode, it
+logs a user out. In 'check' mode it determines the identification status
+of the user using cookies.
 
 Possible user identification status are:
 
@@ -54,11 +57,14 @@ Template to display if user has been identified.
 
 =item * hard_logout
 
-If 'true' in logout mode, this parameter not only unverifies the user, but erases identification cookies too. The default is to retain identified status.
+If 'true' in logout mode, this parameter not only unverifies the
+user, but erases identification cookies too. The default is to retain
+identified status.
 
 =item * stop
 
-Directive indicating that if a specified template is displayed, the remainder of the current template must not be displayed.
+Directive indicating that if a specified template is displayed, the
+remainder of the current template must not be displayed.
 
 =back
 
@@ -75,13 +81,14 @@ $config hash with all required parameters is presented below:
  customer => { 
     list_uri            => '/Customers', 
     id_cookie           => 'id_customer',    
-    id_cookie_expire    => 126230400,       # (seconds) optional, default is 4y  
+    id_cookie_expire    => 126230400,       # (seconds) optional,
+                                            # default is 10 years
     user_prop           => 'email',         # optional, see below    
     pass_prop           => 'password', 
     pass_encrypt        =>  'md5',          # optional, see below
     vf_key_prop         => 'verify_key',    # optional, see below 
     vf_key_cookie       => 'key_customer',  # optional, see below
-    vf_time_prop        => 'latest_verified_access',    
+    vf_time_prop        => 'verify_time',   # time of last verification
     vf_expire_time      => '600',           # seconds
     cb_uri              => 'IdentifyUser/customer' # optional
  }
@@ -107,6 +114,13 @@ Name attribute of a user object. If there is no 'user_prop' parameter in
 the configuration it is assumed that user ID is the key for the given
 list.
 
+An interesting capability is to specify name as a URI style path, for
+instance if a member has many alternative names that all can be used to
+log-in and these names are stored in a list called Nicknames on each
+member object, then the following might be used:
+
+ user_prop => 'Nicknames/nickname'
+
 =item pass_prop
 
 Password attribute of user object.
@@ -137,6 +151,19 @@ Attribute of user object which stores the time of latest verified access.
 
 Time period for which user remains verified.
 
+Please note, that the cookie with the customer key will be set to expire
+in 10 years and actual expiration will only be checked using the content
+of 'vf_time_prop' field value. The reason for such behavior is that many
+(if not all) versions of Microsoft IE have what can be considered a
+serious bug -- they compare the cookie expiration time to the local time
+on the computer. And therefore if customer computer is accidentally set
+to some future date the cookie might expire immediately and prevent this
+customer from logging into the system at all. Most (if not all) versions
+of Netscape and Mozilla do not have this problem.
+
+Therefore, when possible we do not trust customer's computer to measure
+time for us and do that ourselves.
+
 =item cb_uri
 
 URI of clipboard where IdentifyUser stores identification and
@@ -192,7 +219,7 @@ use XAO::Objects;
 use base XAO::Objects->load(objname => 'Web::Action');
 
 use vars qw($VERSION);
-($VERSION)=(q$Id: IdentifyUser.pm,v 1.10 2002/01/04 02:13:23 am Exp $ =~ /(\d+\.\d+)/);
+($VERSION)=(q$Id: IdentifyUser.pm,v 1.16 2002/11/09 02:22:15 am Exp $ =~ /(\d+\.\d+)/);
 
 ###############################################################################
 
@@ -306,35 +333,35 @@ sub check {
 
         my $current_time=time;
         if($last_vf && $current_time - $last_vf <= $vf_expire_time) {
-            
+
             ##
             # If optional 'vf_key_prop' and 'vf_key_cookie' parameters
             # are present checking the content of the key cookie and
             # appropriate field in the user profile
             #
             if ($config->{vf_key_prop} && $config->{vf_key_cookie}) {
-                my $web_key=$self->cgi->cookie($config->{vf_key_cookie});
-                my $db_key=$user->get($config->{vf_key_prop});
-                if($db_key eq $web_key) {
+                my $web_key=$self->cgi->cookie($config->{vf_key_cookie}) || '';
+                my $db_key=$user->get($config->{vf_key_prop}) || '';
+                if($web_key && $db_key eq $web_key) {
                     $clipboard->put("$cb_uri/verified" => 1);
 
                     ##
                     # In order to reduce global heating we only transfer
-                    # cookie if more then 1/10 of the expiration time
-                    # passed since the last visit.
+                    # cookie if more then 1/20 of the expiration time passed
+                    # since the last visit.
                     #
-                    # Mozilla (and probably other browsers as well)
-                    # seems to re-write its cookies file every time it
-                    # gets a new cookie. Nobody cares, but I don't like
-                    # it for aesthetic reasons.
+                    # Mozilla (and probably other browsers as well) seems
+                    # to re-write its cookies file every time it gets a
+                    # new cookie. Nobody cares, but I don't like it for
+                    # aesthetic reasons.
                     #
-                    my $quant=int($vf_expire_time/10);
+                    my $quant=int($vf_expire_time/20);
                     if($current_time-$last_vf > $quant) {
                         $self->siteconfig->add_cookie(
                             -name    => $config->{vf_key_cookie},
                             -value   => $web_key,
                             -path    => '/',
-                            -expires => '+' . ($vf_expire_time+$quant) . 's',
+                            -expires => '+10y',
                         );
                         $user->put($vf_time_prop => $current_time);
                     }
@@ -389,6 +416,8 @@ sub display_results ($$$;$) {
             CB_URI      => $cb_uri || '',
             ERRSTR      => $errstr || '',
             TYPE        => $type,
+            NAME        => $self->clipboard->get("$cb_uri/name") || '',
+            VERIFIED    => $self->clipboard->get("$cb_uri/verified") || '',
         );
 
         $self->finaltextout('') if $args->{stop};
@@ -465,6 +494,7 @@ sub login ($;%) {
     #
     if($user) {
         my $password=$args->{password};
+
         if(! $password) {
             $errstr="No password given";
         }
@@ -488,6 +518,19 @@ sub login ($;%) {
             if($dbpass ne $password) {
                 $errstr='Password mismatch';
             }
+            else {
+
+                ##
+                # Calling overridable function that can check some
+                # additional condition. Return a string with the
+                # suggested error message or an empty string on success.
+                #
+                $errstr=$self->login_check(name => $username,
+                                           object => $user,
+                                           password => $password,
+                                           type => $type,
+                                          );
+            }
         }
     }
 
@@ -507,7 +550,7 @@ sub login ($;%) {
             -name    => $config->{vf_key_cookie},
             -value   => $random_key,
             -path    => '/',
-            -expires => '+' . $config->{vf_expire_time} . 's',
+            -expires => '+10y',
         );
     }
 
@@ -521,7 +564,8 @@ sub login ($;%) {
     ##
     # Setting user name cookie
     #
-    my $expire=$config->{id_cookie_expire} ? "+$config->{id_cookie_expire}s" : '+4y';
+    my $expire=$config->{id_cookie_expire} ? "+$config->{id_cookie_expire}s"
+                                           : '+10y';
     my $id_cookie=$config->{id_cookie} ||
         throw XAO::E::DO::Web::IdentifyUser "login - no 'id_cookie' in the configuration";
     $self->siteconfig->add_cookie(
@@ -548,6 +592,32 @@ sub login ($;%) {
 
 ##############################################################################
 
+=item login_check ()
+
+A method that can be overriden in a derived object to check addition
+conditions for letting a user in. Get the following arguments as its
+input:
+
+ name       => name of user object
+ password   => password
+ object     => reference to a database object containing user info
+ type       => user type
+
+This method is called after all standard checks - it is guaranteed that
+user object exists and password matches its database record.
+
+Must return empty string on success or suggested error message on
+failure. That error message will be passed in ERRSTR argument to the
+templates.
+
+=cut
+
+sub login_check ($%) {
+    return '';
+}
+
+##############################################################################
+
 =item logout ()
 
 Logs out user. Resetting vf_time_prop database field and clearing
@@ -600,7 +670,7 @@ sub logout{
             -name    => $config->{vf_key_cookie},
             -value   => '0',
             -path    => '/',
-            -expires => '-1s',
+            -expires => 'now',
         );
 
         $user->delete($config->{vf_key_prop}) if $user;
@@ -620,7 +690,7 @@ sub logout{
             -name    => $config->{id_cookie},
             -value   => '0'
             -path    => '/',
-            -expires => '-1s',
+            -expires => 'now',
         );
 
         return $self->display_results($args,'anonymous');
