@@ -36,6 +36,21 @@ sub test_expand {
                        TEST => 'TEST<>?');
     $self->assert($got eq 'system:[[TEST<>?]{TEST&lt;&gt;?}]',
                   "Got wrong value for /local.txt: $got");
+
+    my %matrix=(
+        '123' => {
+            template => q(<%Page
+                            template={'<%Page template="<%TEST%>"%>'}
+                            TEST='123'
+                          %>),
+        },
+    );
+    foreach my $expect (keys %matrix) {
+        my $args=$matrix{$expect};
+        my $got=$page->expand($args);
+        $self->assert($got eq $expect,
+                      "Expected '$expect', got '$got'");
+    }
 }
 
 sub test_fs {
@@ -102,4 +117,49 @@ sub test_throw {
                   "Page::throw error - $error");
 }
 
+sub test_cache {
+    my $self=shift;
+
+    my $page=XAO::Objects->new(objname => 'Web::MyPage');
+    $self->assert(ref($page),
+                  "Can't load MyPage object");
+
+    my $cache_val=123;
+    my $cache_sub=sub { return $cache_val++ };
+
+    my $cache=$page->cache(
+        name        => 'test',
+        retrieve    => $cache_sub,
+        coords      => 'name',
+        expire      => 60,
+    );
+    $self->assert(ref($cache),
+                  "Can't load Cache object");
+
+    my $got=$cache->get(name => 'foo');
+    $self->assert($got == 123,
+                  "Wrong value from cache, expected 123, got $got");
+
+    $got=$cache->get(name => 'foo');
+    $self->assert($got == 123,
+                  "Wrong value from cache, expected 123, got $got");
+
+    my $page1=XAO::Objects->new(objname => 'Web::MyPage');
+    $self->assert(ref($page),
+                  "Can't load MyPage object");
+
+     my $cache1=$page1->cache(
+        name        => 'test',
+        retrieve    => $cache_sub,
+        coords      => 'name',
+        expire      => 60,
+    );
+    $self->assert(ref($cache1),
+                  "Can't load Cache object (Page1)");
+
+    $got=$cache1->get(name => 'foo');
+    $self->assert($got == 123,
+                  "Wrong value from cache, expected 123, got $got");
+}
+
 1;
